@@ -12,7 +12,6 @@ from narwhals._arrow.utils import parse_datetime_format
 from narwhals._pandas_like.utils import get_dtype_backend
 from tests.utils import (
     PANDAS_VERSION,
-    PYARROW_VERSION,
     assert_equal_data,
     is_pyarrow_windows_no_tzdata,
     is_windows,
@@ -205,9 +204,6 @@ def test_pyarrow_infer_datetime_raise_inconsistent_date_fmt(
 def test_to_datetime_tz_aware(
     constructor: Constructor, request: pytest.FixtureRequest, format: str | None
 ) -> None:
-    if "pyarrow_table" in str(constructor) and PYARROW_VERSION < (13,):
-        # bugged
-        pytest.skip()
     if is_pyarrow_windows_no_tzdata(constructor) or (
         "sqlframe" in str(constructor) and format is not None and is_windows()
     ):
