@@ -3,7 +3,7 @@ from __future__ import annotations
 import pytest
 
 import narwhals as nw
-from tests.utils import PYARROW_VERSION, Constructor, ConstructorEager, assert_equal_data
+from tests.utils import Constructor, ConstructorEager, assert_equal_data
 
 
 @pytest.mark.parametrize(
@@ -24,9 +24,6 @@ from tests.utils import PYARROW_VERSION, Constructor, ConstructorEager, assert_e
 def test_str_to_uppercase(
     constructor: Constructor, data: dict[str, list[str]], expected: dict[str, list[str]]
 ) -> None:
-    if "dask" in str(constructor) and PYARROW_VERSION < (12,):
-        pytest.skip()
-
     if (
         any(
             x in str(constructor)
