@@ -1,6 +1,9 @@
 from __future__ import annotations
 
+import datetime as dt
 import os
+import re
+from decimal import Decimal
 from typing import TYPE_CHECKING, Any
 
 import pandas as pd
@@ -8,9 +11,10 @@ import pyarrow as pa
 import pytest
 
 import narwhals as nw
-from narwhals.exceptions import ComputeError, InvalidOperationError
+from narwhals.exceptions import ComputeError, DuplicateError, InvalidOperationError
 from tests.utils import (
     PANDAS_VERSION,
+    POLARS_VERSION,
     PYARROW_VERSION,
     Constructor,
     ConstructorEager,
@@ -20,6 +24,9 @@ from tests.utils import (
 if TYPE_CHECKING:
     from collections.abc import Mapping
 
+    from narwhals.typing import NonNestedLiteral
+
+
 data: Mapping[str, Any] = {"a": [1, 1, 3], "b": [4, 4, 6], "c": [7.0, 8.0, 9.0]}
 
 df_pandas = pd.DataFrame(data)
@@ -86,6 +93,16 @@ def test_group_by_iter(constructor_eager: ConstructorEager) -> None:
     assert sorted(keys) == sorted(expected_keys)
 
 
+def test_group_by_iter_non_str_pandas() -> None:
+    expected = {"a": {0: [1], 1: ["a"]}, "b": {0: [2], 1: ["b"]}}
+    df = nw.from_native(pd.DataFrame({0: [1, 2], 1: ["a", "b"]}))
+    groups: dict[Any, Any] = {keys[0]: df for keys, df in df.group_by(1)}  # type: ignore[call-overload]
+    assert groups.keys() == {"a", "b"}
+    groups["a"] = groups["a"].to_dict(as_series=False)
+    groups["b"] = groups["b"].to_dict(as_series=False)
+    assert_equal_data(groups, expected)
+
+
 def test_group_by_nw_all(constructor: Constructor) -> None:
     df = nw.from_native(constructor({"a": [1, 1, 2], "b": [4, 5, 6], "c": [7, 8, 9]}))
     result = df.group_by("a").agg(nw.all().sum()).sort("a")
@@ -178,7 +195,10 @@ def test_group_by_n_unique_w_missing(constructor: Constructor) -> None:
 
 def test_group_by_same_name_twice() -> None:
     df = pd.DataFrame({"a": [1, 1, 2], "b": [4, 5, 6]})
-    with pytest.raises(ValueError, match="Expected unique output names"):
+    pattern = re.compile(
+        "expected unique.+names.+'b'.+2 times", re.IGNORECASE | re.DOTALL
+    )
+    with pytest.raises(DuplicateError, match=pattern):
         nw.from_native(df).group_by("a").agg(nw.col("b").sum(), nw.col("b").n_unique())
 
 
@@ -317,9 +337,8 @@ def test_group_by_shift_raises(constructor: Constructor) -> None:
 def test_double_same_aggregation(
     constructor: Constructor, request: pytest.FixtureRequest
 ) -> None:
-    if any(x in str(constructor) for x in ("dask", "modin", "cudf")):
+    if any(x in str(constructor) for x in ("dask", "cudf")):
         # bugged in dask https://github.com/dask/dask/issues/11612
-        # and modin lol https://github.com/modin-project/modin/issues/7414
         # and cudf https://github.com/rapidsai/cudf/issues/17649
         request.applymarker(pytest.mark.xfail)
     df = nw.from_native(constructor({"a": [1, 1, 2], "b": [4, 5, 6]}))
@@ -331,9 +350,8 @@ def test_double_same_aggregation(
 def test_all_kind_of_aggs(
     constructor: Constructor, request: pytest.FixtureRequest
 ) -> None:
-    if any(x in str(constructor) for x in ("dask", "cudf", "modin")):
+    if any(x in str(constructor) for x in ("dask", "cudf")):
         # bugged in dask https://github.com/dask/dask/issues/11612
-        # and modin lol https://github.com/modin-project/modin/issues/7414
         # and cudf https://github.com/rapidsai/cudf/issues/17649
         request.applymarker(pytest.mark.xfail)
     if "pandas" in str(constructor) and PANDAS_VERSION < (1, 4):
@@ -523,3 +541,81 @@ def test_renaming_edge_case(constructor: Constructor) -> None:
     result = nw.from_native(constructor(data)).group_by(nw.col("a")).agg(nw.all().min())
     expected = {"a": [0], "_a_tmp": [1], "b": [4]}
     assert_equal_data(result, expected)
+
+
+def test_group_by_len_1_column(
+    constructor: Constructor, request: pytest.FixtureRequest
+) -> None:
+    """Based on a failure from marimo.
+
+    - https://github.com/marimo-team/marimo/blob/036fd3ff89ef3a0e598bebb166637028024f98bc/tests/_plugins/ui/_impl/tables/test_narwhals.py#L1098-L1108
+    - https://github.com/marimo-team/marimo/blob/036fd3ff89ef3a0e598bebb166637028024f98bc/marimo/_plugins/ui/_impl/tables/narwhals_table.py#L163-L188
+    """
+    if any(x in str(constructor) for x in ("dask",)):
+        # `dask`
+        #     ValueError: conflicting aggregation functions: [('size', 'a'), ('size', 'a')]
+        request.applymarker(pytest.mark.xfail)
+    data = {"a": [1, 2, 1, 2, 3, 4]}
+    expected = {"a": [1, 2, 3, 4], "len": [2, 2, 1, 1], "len_a": [2, 2, 1, 1]}
+    result = (
+        nw.from_native(constructor(data))
+        .group_by("a")
+        .agg(nw.len(), nw.len().alias("len_a"))
+        .sort("a")
+    )
+    assert_equal_data(result, expected)
+
+
+@pytest.mark.parametrize(
+    ("low", "high"),
+    [
+        ("A", "B"),
+        (1.5, 5.2),
+        (dt.datetime(2000, 1, 1), dt.datetime(2002, 1, 1)),
+        (dt.date(2000, 1, 1), dt.date(2002, 1, 1)),
+        (dt.time(5, 0, 0), dt.time(14, 0, 0)),
+        (dt.timedelta(32), dt.timedelta(800)),
+        (False, True),
+        (b"a", b"z"),
+        (Decimal("43.954"), Decimal("264.124")),
+    ],
+    ids=[
+        "str",
+        "float",
+        "datetime",
+        "date",
+        "time",
+        "timedelta",
+        "bool",
+        "bytes",
+        "Decimal",
+    ],
+)
+def test_group_by_no_preserve_dtype(
+    constructor_eager: ConstructorEager, low: NonNestedLiteral, high: NonNestedLiteral
+) -> None:
+    """Minimal repro for [`px.sunburst` failure].
+
+    The issue appeared for `n_unique`, but applies for any [aggregation that requires a function].
+
+    [`px.sunburst` failure]: https://github.com/narwhals-dev/narwhals/pull/2680#discussion_r2151972940
+    [aggregation that requires a function]: https://github.com/pandas-dev/pandas/issues/57317
+    """
+    if (
+        "polars" in str(constructor_eager)
+        and isinstance(low, Decimal)
+        and POLARS_VERSION < (1, 21, 0)
+    ):
+        pytest.skip("Decimal support in group_by for polars didn't stabilize until 1.0.0")
+    data = {
+        "col_a": ["A", "B", None, "A", "A", "B", None],
+        "col_b": [low, low, high, high, None, None, None],
+    }
+    expected = {"col_a": [None, "A", "B"], "n_unique": [2, 3, 2]}
+    frame = nw.from_native(constructor_eager(data))
+    result = (
+        frame.group_by("col_a").agg(n_unique=nw.col("col_b").n_unique()).sort("col_a")
+    )
+    actual_dtype = result.schema["n_unique"]
+    assert actual_dtype.is_integer()
+    assert_equal_data(result, expected)
